`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: BITSILICA PRIVATE LIMITED
// Engineer: ABHIGNA.B
// 
// Create Date: 19/02/2021 10:47:19 AM
// Design Name: ENCODER FROZEN BIT INSERTION
// Module Name: encoder_frozen_bit_insertion_tb
// Project Name: POLAR ENCODER-DECODER
// Target Devices: Zynq UltraScale+ ZCU111 Evaluation Platform (xczu28dr-ffvg1517-2-e)
// Tool Versions: VIVADO 2020.1
// Description: TB code to verify RTL for inserting frozen bits into message, according to given reliability sequence.
// 
// Dependencies: 
// 
// Revision:2.0
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

// encoder_frozen_bit_insertion_tb is written to verify encoder_frozen_bit_insertion_rtl
module encoder_frozen_bit_insertion_tb();

//parameter N = 32; //12;                               // length of information+CRC+Frozen bits
//parameter K = 16;  //4;                                // length of information+CRC bits
    
reg           clock_i  = 1'b0;                  // system clock
reg           reset_ni = 1'b1;                  // active low synchronous reset
reg      [9:0]N_in;    
reg      [8:0]K_in; 
//reg     [K-1:0]  msg_i;                            // input information+CRC bits
reg     [163:0]  msg_i;                            // input information+CRC bits
//wire [9:0]    pc_pos1_o,pc_pos2_o,pc_pos3_o;    // first, second, third positions to insert parity check bits
//wire [N-1:0]  msg_o;                            // information+CRC+frozen bits

 // wire [9:0]   index_out32;
 // wire [9:0]   index_out64;
 // wire [9:0]  index_out128;
 // wire [9:0]  index_out256;
 // wire [9:0]  index_out512;
   wire [511:0]msg_1_out;
  // Instantiating and connecting the design module with test bench
  encoder_frozen_bit_insertion_rtl //#(
  //  .N(N),
  //  .K(K)
 // )
  DUT(
    .clock_i(clock_i),
    .reset_ni(reset_ni),
    .msg_i(msg_i),
    .N_in (N_in),
    .K_in (K_in),
    //.pc_pos1_o(pc_pos1_o),
    //.pc_pos2_o(pc_pos2_o),
    //.pc_pos3_o(pc_pos3_o),
//    . msg_o(msg_o),
    //.  index_out32 ( index_out32), 
   // .  index_out64(index_out64),
    // . index_out128(index_out128),
    //.  index_out256(index_out256),
   //  . index_out512(index_out512),
     .msg_1_out(msg_1_out) );
     
     
  always #2.03 clock_i = !clock_i;      // clock generation

  initial begin
    @(negedge clock_i);           // waiting for the negedge of clock_i
    reset_ni = 'd0;               // activating the reset_ni
    @(negedge clock_i);
    reset_ni = 'd1;               // deactivating the reset_ni
    N_in = 'd32;
    K_in = 'd16;
    @(negedge clock_i);
    msg_i = 'hC8F5;  //aaaabbbbccccddddaaaabbbbccccddddeeeeffff111122223333444455556666; // applying inputs to the DUT
    #81
    
   @(negedge clock_i);           // waiting for the negedge of clock_i
    reset_ni = 'd0;               // activating the reset_ni
    @(negedge clock_i);
    #4 reset_ni = 'd1; 
    
    N_in = 'd64;
    K_in = 'd20;
    @(negedge clock_i);
    msg_i = 'h0001;    //'h3; 
    //reset_ni = 'd0;
    #140
    
   @(negedge clock_i);           // waiting for the negedge of clock_i
     reset_ni = 'd0;               // activating the reset_ni
    @(negedge clock_i);
    #4 reset_ni = 'd1; 
    
    
    @(negedge clock_i);
    N_in = 'd128;
    K_in = 'd12;
    @(negedge clock_i);
    msg_i = 'h9C83;  //4'b1101;
    #300 $finish;
  end
  
endmodule


